//<c>   Copyright 1997-1998 by Gerry J. Danen; all rights reserved.

//<t>   Check Upload info in Maximus log file

//      Compiled with Microsoft C 8.00. Requires DANEN library to recompile.
//      Source released into the Public Domain.


#include"gd_tools.h"


#define pgVERS_ "V2.00"
#define pgTITL_ "Collect Data from Maximus Log"
#define pgCOPR_ "1997"


UCHAR   iniFile[FS_LEN], iFile[FS_LEN], oFil1[]="MAX-DL.LOG",
        oFil2[]="MAX-UL.LOG", oFil3[]="MAX-NU.LOG", oFil4[]="MAX-LOG.LOG",
        old[1001], work[1001], *w, sysopName[81], ulFmt[81], stopExt[11][11] ;

FILE    *iFh, *oF1, *oF2, *oF3, *oF4 ;
long    lines = 0 ;
int     iDateFmt = 1 ;                          // 1=std, 2=short
int     iNameSize = 30 ;                        // length of user name
int     iExt2Stop = 0 ;                         // reflects entries in stopExt[][]


void    show_help( void )
{
    logon(1);
    fprintf( stderr, "\n%s collects information from the Maximus log,\n"
                     "and appends it to these log files:\n"
                     "\t\tDownloads: %s\n"
                     "\t\tUploads:   %s\n"
                     "\t\tNew Users: %s\n"
                     "\t\tLogins:    %s\n"
                     "\nSyntax:  %s input_logfile\n\n",
                     get_global_program_name(), oFil1, oFil2, oFil3, oFil4,
                     get_global_program_name() );
    exit( GD_ERROR_HELP );
}


void    get_filesize( UCHAR buf[] )
{
    static UCHAR s[81], w[81] ;

//  + 20 Feb 09:14:20 MAX  CPS: 1392 (39001 bytes)  Efficiency: 96%
    buf[0] = EOS ;
    ST_CPY( s, &old[23] );
    st_remove_word( s );                        // remove "CPS:"
    st_remove_word( s );                        // remove cps
    st_get_word( w, s );                        // bytes with leading "("
    if ( w[0] == '(' )
    {
        w[0] = ' ' ;
        st_trim( w );
        strcpy( buf, w );
    }
}


void    fix_date( UCHAR buf[] )
{
    static UCHAR d[9], m[9], t[9] ;
    static int   id, im ;

    st_trim( buf );                             // 20 Feb 09:14:20
    switch ( iDateFmt )                         // 1=std, 2=short
    {
        case 1 :
            return ;
        case 2 :
            st_get_word( d, buf );              // get day
            st_remove_word( buf );              // remove day
            st_get_word( m, buf );              // get month
            st_remove_word( buf );              // remove month
            buf[2] = buf[5] = ' ' ;
            st_remove_spaces( buf );
            ST_CPY( t, buf );
            id = atoi(d);
            im = Month2Number(m);
            sprintf( buf, "%02d%02d %s", im, id, t );
            return ;
        default :
            return ;
    }
}


BOOL    stop_ext( UCHAR fn[] )
{
    int         x ;

    if ( ! iExt2Stop )                          // nothing excluded
        return( FALSE );

    LOOP( x, 0, (iExt2Stop-1), 1 )              // see if we have a match
    {
        if ( st_eq(&fn[strlen(fn)-strlen(stopExt[x])], stopExt[x]) )
            return( TRUE );
    }
    return( FALSE );
}


void    main_init( int argc, char *argv[] )
{
    int         x, xcnt=0 ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);

    strcpy( iniFile, argv[0] );                 // create .ini file name
    {
    char *pIni ;
    pIni = strchr( iniFile, '.' );
    if ( pIni != NULL )
        *pIni = EOS ;
    }
    strcat( iniFile, ".INI" );
    fprintf( stderr, "\n%s: Using setup from %s\nOptions:\n", get_global_program_name(), iniFile );

//  sysop name does not show in upload/download logs
    fl_parse_ini( iniFile, "MaxCheck", "sysop", sysopName, &xcnt );

//  length of user name in upload/download logs
    fl_parse_ini( iniFile, "MaxCheck", "namesize", work, &xcnt );
    iNameSize = atoi(work);
    if ( iNameSize < 10 || iNameSize > 50 )
        iNameSize = 20 ;
    sprintf( ulFmt, "%s%d.%d%s\n", "%s %c %-", iNameSize, iNameSize, "s%9s %s" );

//  std=as shown in Maximus log,
//  short=MMDD HHMMSS
    fl_parse_ini( iniFile, "MaxCheck", "datefmt", work, &xcnt );
    if ( st_eq(work, "std") )                   // 1=std, 2=short
        iDateFmt = 1 ;
    if ( st_eq(work, "short") )
        iDateFmt = 2 ;

//  these filename extentions will not show in upload/download logs
//  excludeExt=.qwk .rep
    fl_parse_ini( iniFile, "MaxCheck", "excludeext", work, &xcnt );
    LOOP( x, 0, 10, 1 )                         // clear entries first
    {
        strcpy( stopExt[x], "" );
    }
    strupr( work );
    LOOP( x, 0, 10, 1 )                         // now see what we have
    {
        if ( strlen(work) )
        {
            st_get_word( old, work );           // get ext
            st_remove_word( work );             // remove ext
            st_cpy( stopExt[iExt2Stop], old, 11 );  // store ext
            iExt2Stop ++ ;
        }
    }
    if ( strlen(sysopName) )
        fprintf( stderr, "\tSkipping ul/dl entries for sysop: %s\n", sysopName );
    if ( iExt2Stop )
    {
        fprintf( stderr, "\tSkipping ul/dl file extensions: " );
        LOOP( x, 0, (iExt2Stop-1), 1 )
        {
            fprintf( stderr, "%s ", stopExt[x] );
        }
        fprintf( stderr, "\n" );
    }

    fprintf( stderr, "\tLength of user name in upload/download logs: %d\n",
                     iNameSize );
    fprintf( stderr, "\tDate format: " );
    switch ( iDateFmt )                         // 1=std, 2=short
    {
        case 1 :
            fprintf( stderr, "standard\n" );
            break ;
        case 2 :
            fprintf( stderr, "short (MMDD HHMMSS)\n" );
            break ;
        default :
            fprintf( stderr, "default\n" );
            break ;
    }

    if ( argc != 2 )
        show_help();

    st_cpy( iFile, (UCHAR *) argv[1], FS_LEN );
    strupr( (char *) iFile );

    if ( ( iFh = fopen( iFile, "r" ) ) == NULL )
    {
        logon(1);
        fprintf( stderr, "\n\nFile %s not found\a\n", iFile );
        show_help();
    }
    if ( ( oF1 = fopen( oFil1, "a" ) ) == NULL )
    {
        logon(1);
        fprintf( stderr, "\n\nCould not append to file %s\a\n", oFil1 );
        show_help();
    }
    if ( ( oF2 = fopen( oFil2, "a" ) ) == NULL )
    {
        logon(1);
        fprintf( stderr, "\n\nCould not append to file %s\a\n", oFil2 );
        show_help();
    }
    if ( ( oF3 = fopen( oFil3, "a" ) ) == NULL )
    {
        logon(1);
        fprintf( stderr, "\n\nCould not append to file %s\a\n", oFil3 );
        show_help();
    }
    if ( ( oF4 = fopen( oFil4, "a" ) ) == NULL )
    {
        logon(1);
        fprintf( stderr, "\n\nCould not append to file %s\a\n", oFil4 );
        show_help();
    }
}


void    main( int argc, char *argv[] )
{
    UCHAR       c1='?', Name[51]="", DTstamp[16], bytes[21] ;
    BOOL        hadLogin = FALSE ;
    int         iStampSize = -1 ;

    main_init( argc, argv );

    fprintf( stderr, "%s: Reading %s\n", get_global_program_name(), iFile );

    FOREVER
    {
        w = fgets( work, (sizeof(work)-1), iFh );
        if ( w )
        {
            lines += 1 ;
            if ( (lines MOD 10) == 0L )
                fprintf( stderr, "\r%ld", lines );

            st_trim( work );
            c1 = work[0] ;
            work[0] = ' ' ;
            ST_CPY( DTstamp, &work[2] );
            fix_date( DTstamp );
            switch ( c1 )
            {
                case '+' :
                    //  + 13 Feb 10:45:10 MAX  Ed Barth calling
                    //  + 13 Feb 16:50:44 MAX  Ilse Kinshella off-line. Calls=32, Len=1, Today=1
                    w = strrchr( work, 'c' );
                    if ( st_eq(w, "calling") )  // save caller name
                    {
                        if (iStampSize == -1 )
                            iStampSize = strlen(DTstamp) ;
                        if ( hadLogin )
                            fprintf( oF4, "! %*s %s did not logout properly...\n",
                                          iStampSize, "", Name );
                        *w = EOS ;
                        ST_CPY( Name, &work[23] );
                        st_trim( Name );
                        fprintf( oF4, "+ %s %s\n", DTstamp, Name );
                        hadLogin = TRUE ;
                    }
                    else
                    {
                        w = strstr( work, "off-line." );
                        if ( w )
                        {
                            *w = EOS ;
                            st_rtrim( work );
                            fprintf( oF4, "- %s %s\n", DTstamp, &work[23] );
                            hadLogin = FALSE ;
                        }
                    }
                    strcpy( old, work );

                case ':' :
                    //  : 13 Feb 10:45:10 MAX  Application
                    if ( st_eq(&work[23], "Application") )
                    {
                        fprintf( oF3, "%s %s\n", DTstamp, Name );
                    }

                case '=' :                      // up/downloads
                    //  = 14 Feb 14:26:12 MAX  UL-Z D:\FIL\SYTMP\MSG01.TXT
                    //  = 15 Feb 15:07:51 MAX  DL-Z D:\FIL\UTARC\WZ32V62.ZIP
                    strupr( &work[22] );
                    strlwr( &work[28] );
                    if ( (work[23]=='D' && work[24]=='L' && work[25]=='-') ||
                         (work[23]=='U' && work[24]=='L' && work[25]=='-') )
                    {
                        //  skip .REP and .QWK records & sysop UL/DL
                        if ( ! (stop_ext(work) || st_eq(Name, sysopName)) )
                        {
                            get_filesize( bytes );
                            //  14 Feb 14:26:12 Z username D:\FIL\SYTMP\MSG01.TXT
                            if ( work[23] == 'D' )
                                fprintf( oF1, ulFmt, DTstamp, work[26], Name,
                                                     bytes, &work[28] );
                            if ( work[23] == 'U' )
                                fprintf( oF2, ulFmt, DTstamp, work[26], Name,
                                                     bytes, &work[28] );
                        }
                    }
            }
        }
        if ( feof(iFh) )
        {
            // check if last one logged out properly
            if ( hadLogin )
                fprintf( oF4, "! %*s %s did not logout properly...\n",
                              iStampSize, "", Name );
            // clean up
            _fcloseall();
            fprintf( stderr, "\rProcessed %ld lines%c\n", lines, (lines>1000L ? BELL : SPACE) );
            exit( GD_ERROR_NONE );
        }
    }
}
